@@ -92,6 +92,7 @@ orc_test_gcc_compile (OrcProgram *p)
   ret = system (cmd);
   if (ret != 0) {
     ORC_ERROR ("gcc failed");
+    printf("%s\n", orc_program_get_asm_code (p));
     return ORC_TEST_FAILED;
   }
 
@@ -145,6 +146,217 @@ orc_test_gcc_compile (OrcProgram *p)
 }
 
 
+#define PREFIX "/opt/arm-2008q3/bin/arm-none-linux-gnueabi-"
+
+OrcTestResult
+orc_test_gcc_compile_neon (OrcProgram *p)
+{
+  char cmd[300];
+  char *base;
+  char source_filename[100];
+  char obj_filename[100];
+  char dis_filename[100];
+  char dump_filename[100];
+  char dump_dis_filename[100];
+  int ret;
+  FILE *file;
+  OrcCompileResult result;
+  OrcTarget *target;
+  unsigned int flags;
+
+  base = "temp-orc-test";
+
+  sprintf(source_filename, "%s-source.s", base);
+  sprintf(obj_filename, "%s.o", base);
+  sprintf(dis_filename, "%s-source.dis", base);
+  sprintf(dump_filename, "%s-dump.bin", base);
+  sprintf(dump_dis_filename, "%s-dump.dis", base);
+
+  target = orc_target_get_by_name ("neon");
+  flags = orc_target_get_default_flags (target);
+  flags |= ORC_TARGET_NEON_CLEAN_COMPILE;
+
+  result = orc_program_compile_full (p, target, flags);
+  if (!ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
+    return ORC_TEST_INDETERMINATE;
+  }
+
+  fflush (stdout);
+
+  file = fopen (source_filename, "w");
+  fprintf(file, "%s", orc_program_get_asm_code (p));
+  fclose (file);
+
+  file = fopen (dump_filename, "w");
+  ret = fwrite(p->code, p->code_size, 1, file);
+  fclose (file);
+
+  sprintf (cmd, PREFIX "gcc -march=armv6t2 -mcpu=cortex-a8 -mfpu=neon -Wall "
+      "-c %s -o %s", source_filename, obj_filename);
+  ret = system (cmd);
+  if (ret != 0) {
+    ORC_ERROR ("arm gcc failed");
+    return ORC_TEST_INDETERMINATE;
+  }
+
+  sprintf (cmd, PREFIX "objdump -dr %s >%s", obj_filename, dis_filename);
+  ret = system (cmd);
+  if (ret != 0) {
+    ORC_ERROR ("objdump failed");
+    return ORC_TEST_INDETERMINATE;
+  }
+
+  sprintf (cmd, PREFIX "objcopy -I binary "
+      "-O elf32-littlearm -B arm "
+      "--rename-section .data=.text "
+      "--redefine-sym _binary_temp_orc_test_dump_bin_start=%s "
+      "%s %s", p->name, dump_filename, obj_filename);
+  ret = system (cmd);
+  if (ret != 0) {
+    printf("objcopy failed\n");
+    return ORC_TEST_INDETERMINATE;
+  }
+
+  sprintf (cmd, PREFIX "objdump -Dr %s >%s", obj_filename, dump_dis_filename);
+  ret = system (cmd);
+  if (ret != 0) {
+    printf("objdump failed\n");
+    return ORC_TEST_INDETERMINATE;
+  }
+
+  sprintf (cmd, "diff -u %s %s", dis_filename, dump_dis_filename);
+  ret = system (cmd);
+  if (ret != 0) {
+    printf("diff failed\n");
+    return ORC_TEST_FAILED;
+  }
+
+  remove (source_filename);
+  remove (obj_filename);
+  remove (dis_filename);
+  remove (dump_filename);
+  remove (dump_dis_filename);
+
+  return ORC_TEST_OK;
+}
+
+#define C64X_PREFIX "/opt/TI/TI_CGT_C6000_6.1.12/bin/"
+
+OrcTestResult
+orc_test_gcc_compile_c64x (OrcProgram *p)
+{
+  char cmd[300];
+  char *base;
+  char source_filename[100];
+  char obj_filename[100];
+  char dis_filename[100];
+  char dump_filename[100];
+  char dump_dis_filename[100];
+  int ret;
+  FILE *file;
+  OrcCompileResult result;
+  OrcTarget *target;
+  unsigned int flags;
+
+  base = "temp-orc-test";
+
+  sprintf(source_filename, "%s-source.s", base);
+  sprintf(obj_filename, "%s-source.obj", base);
+  sprintf(dis_filename, "%s-source.dis", base);
+  sprintf(dump_filename, "%s-dump.bin", base);
+  sprintf(dump_dis_filename, "%s-dump.dis", base);
+
+  target = orc_target_get_by_name ("c64x");
+  flags = orc_target_get_default_flags (target);
+
+  result = orc_program_compile_full (p, target, flags);
+  if (!ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
+    return ORC_TEST_INDETERMINATE;
+  }
+
+  fflush (stdout);
+
+  file = fopen (source_filename, "w");
+  fprintf(file, "%s", orc_program_get_asm_code (p));
+  fclose (file);
+
+  file = fopen (dump_filename, "w");
+  ret = fwrite(p->code, p->code_size, 1, file);
+  fclose (file);
+
+  sprintf (cmd, C64X_PREFIX "cl6x -mv=6400+ "
+      "-c %s", source_filename);
+  ret = system (cmd);
+  if (ret != 0) {
+    ORC_ERROR ("compiler failed");
+    //printf("%s\n", orc_program_get_asm_code (p));
+    return ORC_TEST_INDETERMINATE;
+  }
+
+  sprintf (cmd, C64X_PREFIX "dis6x %s >%s", obj_filename, dis_filename);
+  ret = system (cmd);
+  if (ret != 0) {
+    ORC_ERROR ("objdump failed");
+    return ORC_TEST_INDETERMINATE;
+  }
+
+#if 0
+  sprintf (cmd, C64X_PREFIX "objcopy -I binary "
+      "-O elf32-littlearm -B arm "
+      "--rename-section .data=.text "
+      "--redefine-sym _binary_temp_orc_test_dump_bin_start=%s "
+      "%s %s", p->name, dump_filename, obj_filename);
+  ret = system (cmd);
+  if (ret != 0) {
+    printf("objcopy failed\n");
+    return ORC_TEST_FAILED;
+  }
+#endif
+
+#if 0
+  sprintf (cmd, C64X_PREFIX "dis6x %s >%s", dump_filename, dump_dis_filename);
+  ret = system (cmd);
+  if (ret != 0) {
+    printf("objdump failed\n");
+    return ORC_TEST_FAILED;
+  }
+
+  sprintf (cmd, "diff -u %s %s", dis_filename, dump_dis_filename);
+  ret = system (cmd);
+  if (ret != 0) {
+    printf("diff failed\n");
+    return ORC_TEST_FAILED;
+  }
+#endif
+
+  remove (source_filename);
+  remove (obj_filename);
+  remove (dis_filename);
+  remove (dump_filename);
+  remove (dump_dis_filename);
+
+  return ORC_TEST_OK;
+}
+
+void
+orc_test_random_bits (void *data, int n_bytes)
+{
+#if 1
+  uint8_t *d = data;
+  int i;
+  for(i=0;i<n_bytes;i++){
+    d[i] = rand();
+  }
+#endif
+#if 0
+  float *d = data;
+  int i;
+  for(i=0;i<n_bytes/4;i++){
+    d[i] = ((rand() & 0xffff)-32768)*0.01;
+  }
+#endif
+}
+
 int
 print_array_val_signed (OrcArray *array, int i, int j)
 {
@@ -320,9 +532,9 @@ orc_test_compare_output_full (OrcProgram *program, int flags)
       orc_array_set_random (src[i-ORC_VAR_S1], &rand_context);
     } else if (program->vars[i].vartype == ORC_VAR_TYPE_DEST) {
       dest_exec[i-ORC_VAR_D1] = orc_array_new (n, m, program->vars[i].size);
-      orc_array_set_pattern (dest_exec[i], 0xa5);
+      orc_array_set_pattern (dest_exec[i], ORC_OOB_VALUE);
       dest_emul[i-ORC_VAR_D1] = orc_array_new (n, m, program->vars[i].size);
-      orc_array_set_pattern (dest_emul[i], 0xa5);
+      orc_array_set_pattern (dest_emul[i], ORC_OOB_VALUE);
     } else if (program->vars[i].vartype == ORC_VAR_TYPE_PARAM) {
       orc_executor_set_param (ex, i, 2);
     }
@@ -623,9 +835,9 @@ orc_test_performance (OrcProgram *program, int flags)
       orc_array_set_random (src[i-ORC_VAR_S1], &rand_context);
     } else if (program->vars[i].vartype == ORC_VAR_TYPE_DEST) {
       dest_exec[i-ORC_VAR_D1] = orc_array_new (n, m, program->vars[i].size);
-      orc_array_set_pattern (dest_exec[i], 0xa5);
+      orc_array_set_pattern (dest_exec[i], ORC_OOB_VALUE);
       dest_emul[i-ORC_VAR_D1] = orc_array_new (n, m, program->vars[i].size);
-      orc_array_set_pattern (dest_emul[i], 0xa5);
+      orc_array_set_pattern (dest_emul[i], ORC_OOB_VALUE);
     } else if (program->vars[i].vartype == ORC_VAR_TYPE_PARAM) {
       orc_executor_set_param (ex, i, 2);
     }
