@@ -14,6 +14,9 @@
 #ifdef HAVE_CODEMEM_MMAP
 #include <sys/mman.h>
 #endif
+#ifdef HAVE_CODEMEM_VIRTUALALLOC
+#include <windows.h>
+#endif
 
 #include <orc/orcprogram.h>
 #include <orc/orcdebug.h>
@@ -29,26 +32,13 @@ orc_compiler_allocate_codemem (OrcCompiler *compiler)
   int fd;
   int n;
 
-  /* FIXME something combining these two would be ideal */
-#if 1
-  {
-    char filename[32] = "/tmp/orcexecXXXXXX";
-    fd = mkstemp (filename);
-    if (fd == -1) {
-      /* FIXME oh crap */
-      ORC_COMPILER_ERROR (compiler, "failed to create temp file");
-      return;
-    }
-    unlink (filename);
-  }
-#else
   {
     char *filename;
 
-    filename = malloc (strlen ("/tmp/orcexec") +
-        strlen (compiler->program->name) + 1);
-    sprintf(filename, "/tmp/orcexec%s", compiler->program->name);
-    fd = open (filename, O_RDWR | O_CREAT, S_IRWXU);
+    filename = malloc (strlen ("/tmp/orcexec..") +
+        strlen (compiler->program->name) + 6 + 1);
+    sprintf(filename, "/tmp/orcexec.%s.XXXXXX", compiler->program->name);
+    fd = mkstemp (filename);
     if (fd == -1) {
       /* FIXME oh crap */
       ORC_COMPILER_ERROR (compiler, "failed to create temp file");
@@ -57,7 +47,6 @@ orc_compiler_allocate_codemem (OrcCompiler *compiler)
     unlink (filename);
     free (filename);
   }
-#endif
 
   n = ftruncate (fd, SIZE);
 
@@ -79,12 +68,21 @@ orc_compiler_allocate_codemem (OrcCompiler *compiler)
 }
 #endif
 
+#ifdef HAVE_CODEMEM_VIRTUALALLOC
+void
+orc_compiler_allocate_codemem (OrcCompiler *compiler)
+{
+  compiler->program->code = VirtualAlloc(NULL, SIZE, MEM_COMMIT, PAGE_EXECUTE_READWRITE);
+  compiler->program->code_exec = compiler->program->code;
+  compiler->program->code_size = SIZE;
+  compiler->codeptr = compiler->program->code;
+}
+#endif
+
 #ifdef HAVE_CODEMEM_MALLOC
 void
 orc_compiler_allocate_codemem (OrcCompiler *compiler)
 {
-  /* Now you know why Windows has viruses */
-
   compiler->program->code = malloc(SIZE);
   compiler->program->code_exec = compiler->program->code;
   compiler->program->code_size = SIZE;
