@@ -5,6 +5,7 @@
 #endif
 
 #include <orc/orconce.h>
+#include <orc/orcdebug.h>
 
 #if defined(HAVE_THREAD_PTHREAD)
 
@@ -12,6 +13,12 @@
 
 static pthread_mutex_t once_mutex = PTHREAD_MUTEX_INITIALIZER;
 
+void
+_orc_once_init (void)
+{
+  
+}
+
 void
 orc_once_mutex_lock (void)
 {
@@ -30,6 +37,12 @@ orc_once_mutex_unlock (void)
 
 static CRITICAL_SECTION once_mutex;
 
+void
+_orc_once_init (void)
+{
+  InitializeCriticalSection (&once_mutex);
+}
+
 void
 orc_once_mutex_lock (void)
 {
@@ -44,6 +57,11 @@ orc_once_mutex_unlock (void)
 
 #else
 
+void
+_orc_once_init (void)
+{
+}
+
 void
 orc_once_mutex_lock (void)
 {
