@@ -34,6 +34,7 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 /**
  * SECTION:orcutils
@@ -41,7 +42,6 @@
  * @short_description: Orc utility functions
  */
 
-#ifdef unused
 int
 get_file_int (const char *file, int *value)
 {
@@ -169,5 +169,4 @@ get_tag_value (char *s, const char *tag)
 
   return _strndup (colon, end-colon);
 }
-#endif
 
