@@ -19,6 +19,7 @@ void output_code_no_orc (OrcProgram *p, FILE *output);
 void output_code_assembly (OrcProgram *p, FILE *output);
 static void print_exec_header (FILE *output);
 static char * get_barrier (const char *s);
+static const char * my_basename (const char *s);
 
 int verbose = 0;
 int error = 0;
@@ -151,7 +152,6 @@ main (int argc, char *argv[])
 
   n = orc_parse (code, &programs);
 
-
   output = fopen (output_file, "w");
   if (!output) {
     printf("Could not write output file: %s\n", output_file);
@@ -159,10 +159,13 @@ main (int argc, char *argv[])
   }
 
   fprintf(output, "\n");
-  fprintf(output, "/* autogenerated from %s */\n", input_file);
+  fprintf(output, "/* autogenerated from %s */\n", my_basename(input_file));
   fprintf(output, "\n");
 
   if (mode == MODE_IMPL) {
+    fprintf(output, "#ifdef HAVE_CONFIG_H\n");
+    fprintf(output, "#include \"config.h\"\n");
+    fprintf(output, "#endif\n");
     fprintf(output, "#ifndef DISABLE_ORC\n");
     fprintf(output, "#include <orc/orc.h>\n");
     fprintf(output, "#else\n");
@@ -173,6 +176,10 @@ main (int argc, char *argv[])
       fprintf(output, "#include <%s>\n", include_file);
     }
     fprintf(output, "\n");
+    for(i=0;i<n;i++){
+      output_code_header (programs[i], output);
+    }
+    fprintf(output, "\n");
     fprintf(output, "%s", orc_target_get_asm_preamble ("c"));
     fprintf(output, "\n");
     for(i=0;i<n;i++){
@@ -833,6 +840,9 @@ output_code_test (OrcProgram *p, FILE *output)
     }
   }
 
+  fprintf(output, "\n");
+  fprintf(output, "    printf (\"  cycles (backup):   \");\n");
+  fprintf(output, "    orc_test_performance (p, ORC_TEST_FLAGS_BACKUP);\n");
   fprintf(output, "\n");
   fprintf(output, "    ret = orc_test_compare_output_backup (p);\n");
   fprintf(output, "    if (!ret) {\n");
@@ -844,6 +854,9 @@ output_code_test (OrcProgram *p, FILE *output)
   fprintf(output, "      error = TRUE;\n");
   fprintf(output, "    }\n");
   fprintf(output, "\n");
+  fprintf(output, "    printf (\"  cycles (compiled): \");\n");
+  fprintf(output, "    orc_test_performance (p, 0);\n");
+  fprintf(output, "\n");
   fprintf(output, "    orc_program_free (p);\n");
   fprintf(output, "  }\n");
   fprintf(output, "\n");
@@ -873,3 +886,18 @@ output_code_assembly (OrcProgram *p, FILE *output)
 
 }
 
+static const char *
+my_basename (const char *s)
+{
+  const char *ret = s;
+  const char *t;
+
+  t = s;
+  while (t[0] != 0) {
+    if (t[0] == '/') ret = t+1;
+    t++;
+  }
+
+  return ret;
+}
+
